@@ -14,7 +14,7 @@
 #' \deqn{T_1 = x_1 - \mu_1;  T_2 = x_2 - \mu_2}
 #' and \eqn{C_c (\kappa_1, \kappa_2, \kappa_3)} denotes the normalizing constant for the cosine model.
 #'
-#' #' Because \eqn{C_c} involves an infinite series with product of Bessel functions,
+#' Because \eqn{C_c} involves an infinite  alternating series with product of Bessel functions,
 #' if \code{kappa3 < -5} or \code{max(kappa1, kappa2, abs(kappa3)) > 50}, \eqn{C_c} is evaluated
 #' numerically via (quasi) Monte carlo method for
 #' numerical stability. These (quasi) random numbers can be provided through the
@@ -22,7 +22,10 @@
 #' a  (quasi) random number between 0 and 1. Alternatively, if \code{n_qrnd} is
 #' provided (and \code{qrnd} is missing), a two dimensional sobol sequence of size \code{n_qrnd} is
 #' generated via the function \link{sobol} from the R package \code{qrng}. If none of \code{qrnd}
-#' or \code{n_qrnd} is available, a two dimensional sobol sequence of size 1e4 is used.
+#' or \code{n_qrnd} is available, a two dimensional sobol sequence of size 1e4 is used. By default Monte
+#' Carlo approximation is used only if \code{kappa3 < -5} or \code{max(kappa1, kappa2, abs(kappa3)) > 50}.
+#' However, a forced Monte Carlo approximation can be made (irrespective of the choice of \code{kappa1, kappa2} and
+#' \code{kappa3}) by setting \code{force_approx_const = TRUE}. See examples.
 #'
 #' @return \code{dvmcos} gives the density  and \code{rvmcos} generates random deviates.
 #'
@@ -68,6 +71,78 @@
 #' # recycled vectors
 #' rvmcos(n, kappa1, kappa2, kappa3, mu1, mu2)
 #'
+#'
+#' \dontrun{
+#' ## Visualizing (quasi) Monte Carlo based approximations of
+#' ## the normalizing constant through density evaluations.
+#'
+#' # "good" setup, where the analytic formula for C_c can be
+#' # calulated without numerical issues
+#' # kappa1 = 1, kappa2 = 1, kappa3 = -2, mu1 = pi, mu2 = pi
+#'
+#' n_qrnd <-  (1:500)*20
+#' # analytic
+#' good.a <- dvmcos(c(3,3), 1, 1, -2, pi, pi, log=TRUE)
+#' # using quasi Monte Carlo
+#' good.q <- sapply(n_qrnd,
+#'                  function(j)
+#'                    dvmcos(c(3,3), 1, 1, -2, pi, pi,
+#'                           log=TRUE, n_qrnd = j,
+#'                           force_approx_const = TRUE))
+#' # using ordinary Monte Carlo
+#' set.seed(1)
+#' good.r <- sapply(n_qrnd,
+#'                  function(j)
+#'                    dvmcos(c(3,3), 1, 1, -2, pi, pi,
+#'                           log=TRUE,
+#'                           qrnd = matrix(runif(2*j), ncol = 2),
+#'                           force_approx_const = TRUE))
+#'
+#'
+#' plot(n_qrnd, good.q, ylim = range(good.a, good.q, good.r),
+#'      col = "orange", type = "l",
+#'      ylab = "",
+#'      main = "dvmcos(c(3,3), 1, 1, -2, pi, pi, log = TRUE)")
+#' points(n_qrnd, good.r, col = "skyblue", type = "l")
+#' abline(h = good.a, lty = 2, col = "grey")
+#' legend("topright",
+#'        legend = c("Sobol", "Random", "Analytic"),
+#'        col = c("orange", "skyblue", "grey"),
+#'        lty = c(1, 1, 2))
+#'
+#'
+#' # "bad" setup, where the calculating C_c
+#' # numerically using the analytic formula is problematic
+#' # kappa1 = 100, kappa2 = 100, kappa3 = -200, mu1 = pi, mu2 = pi
+#'
+#' n_qrnd <-  (1:500)*20
+#'
+#' # using quasi Monte Carlo
+#' bad.q <- sapply(n_qrnd,
+#'                 function(j)
+#'                   dvmcos(c(3,3), 100, 100, -200, pi, pi,
+#'                          log=TRUE, n_qrnd = j,
+#'                          force_approx_const = TRUE))
+#' # using ordinary Monte Carlo
+#' set.seed(1)
+#' bad.r <- sapply(n_qrnd,
+#'                 function(j)
+#'                   dvmcos(c(3,3), 100, 100, -200, pi, pi,
+#'                          log=TRUE,
+#'                          qrnd = matrix(runif(2*j), ncol = 2),
+#'                          force_approx_const = TRUE))
+#'
+#'
+#' plot(n_qrnd, bad.q, ylim = range(bad.q, bad.r),
+#'      col = "orange", type = "l",
+#'      ylab = "",
+#'      main = "dvmcos(c(3,3), 100, 100, -200, pi, pi, log = TRUE)")
+#' points(n_qrnd, bad.r, col = "skyblue", type = "l")
+#' legend("topright",
+#'        legend = c("Sobol", "Random"),
+#'        col = c("orange", "skyblue"), lty = 1)
+#'}
+#'
 #' @export
 rvmcos <- function(n, kappa1=1, kappa2=1,
                    kappa3=0, mu1=0, mu2=0, method=NULL)
@@ -162,12 +237,12 @@ dvmcos <- function(x, kappa1=1, kappa2=1, kappa3=0, mu1=0,
 
   ell <- list(...)
 
-  if (!is.null(ell$qrnd_grid)) {
-    qrnd_grid <- ell$qrnd_grid
+  if (!is.null(ell$qrnd)) {
+    qrnd_grid <- ell$qrnd
     dim_qrnd <- dim(qrnd_grid)
     if (!is.matrix(qrnd_grid) | is.null(dim_qrnd) |
         dim_qrnd[2] != 2)
-      stop("qrnd_grid must be a two column matrix")
+      stop("\'qrnd\' must be a two column matrix")
     n_qrnd <- dim_qrnd[1]
   } else if (!is.null(ell$n_qrnd)){
     n_qrnd <- round(ell$n_qrnd)
@@ -195,11 +270,24 @@ dvmcos <- function(x, kappa1=1, kappa2=1, kappa3=0, mu1=0,
   n_x <- nrow(x)
 
 
+
+  if (is.null(ell$force_approx_const)) {
+    force_approx_const <- FALSE
+  } else if (!is.logical(ell$force_approx_const)) {
+    stop("\'force_approx_const\' must be logical.")
+  } else {
+    force_approx_const <- ell$force_approx_const
+  }
+
+
   l_const_all <- rep(0, n_par)
+
   for(j in 1:n_par) {
-    if ((kappa3[j] >= -5 & max(kappa1[j], kappa2[j],
-                               abs(kappa3[j])) <= 50) |
-        abs(kappa3[j]) < 1e-4) {
+    if (all(!force_approx_const,
+            ((kappa3[j] >= -5 & max(kappa1[j], kappa2[j],
+                                    abs(kappa3[j])) <= 50) |
+             abs(kappa3[j]) < 1e-4))) {
+
       l_const_all[j] <- log(const_vmcos_anltc(kappa1[j], kappa2[j],
                                               kappa3[j]))
     } else {
